{

    /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
    *                                                                 *
    *   MathiX TPU 1.31 BETA                                           *
    *   Copyright  2002 Sebastian Pipping <webmaster@hartwork.org>   *
    *                                                                 *
    *   --> http://www.hartwork.org                                   *
    *                                                                 *
    *                                                                 *
    *   This source code is released under LGPL.                      *
    *   See LGPL.txt for details.                        2002-10-16   *
    *                                                                 *
    \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

}

UNIT mathix;

{ ----------------------------------------------------------------------------------------------------- INTERFACE ----- }
INTERFACE

  FUNCTION e : REAL;
  FUNCTION sign( x : REAL ) : INTEGER;
  FUNCTION tan( x : REAL ) : REAL;
  FUNCTION cot( x : REAL ) : REAL;
  FUNCTION asin( x : REAL ) : REAL;
  FUNCTION acos( x : REAL ) : REAL;
  FUNCTION atan( x : REAL ) : REAL;
  FUNCTION acot( x : REAL ) : REAL;
  FUNCTION log( x, a : REAL ) : REAL;
  FUNCTION log10( x : REAL ) : REAL;
  FUNCTION pow( a, x : REAL ) : REAL;
  FUNCTION fac( x : REAL ) : REAL;
  FUNCTION sigmoid( x, a : REAL ) : REAL;
  FUNCTION sinh( x : REAL ) : REAL;
  FUNCTION cosh( x : REAL ) : REAL;
  FUNCTION tanh( x : REAL ) : REAL;
  FUNCTION coth( x : REAL ) : REAL;


{ ------------------------------------------------------------------------------------------------ IMPLEMENTATION ----- }
IMPLEMENTATION

  CONST prec = 0.00000000001; { 10^(-12 )}

{ ------------------------------------------------------------------------------------------------------------- e ----- }
  FUNCTION e : REAL;
    BEGIN
      e := EXP( 1 );

    END;


{ ---------------------------------------------------------------------------------------------------------- sign ----- }
  FUNCTION sign( x : REAL ) : INTEGER;
    BEGIN
      IF ( x < 0 ) THEN
        sign := -1
      ELSE
        sign := 1;

    END;


{ ----------------------------------------------------------------------------------------------------------- tan ----- }
  FUNCTION tan( x : REAL ) : REAL;
    BEGIN
      tan := SIN( x ) / COS( x );

    END;


{ ----------------------------------------------------------------------------------------------------------- cot ----- }
  FUNCTION cot( x : REAL ) : REAL;
    BEGIN
      cot := COS( x ) / SIN( x );

    END;


{ ---------------------------------------------------------------------------------------------------------- asin ----- }
  FUNCTION asin( x : REAL ) : REAL;
    VAR
      temp, prev : REAL;
      num, den   : LONGINT;
      i, ni, di  : BYTE;

    BEGIN
      temp := x;
      i := 3;

      num := 1; { Numerator }
      ni  := 1;
      den := 1; { Denominator }
      di  := 2;

      REPEAT
        prev := temp;
        num := num * ni;
        den := den * di;
        temp := temp + EXP( i * LN( x ) ) * num / den / ( di + 1 );
        i := i + 2;
        ni := ni + 2;
        di := di + 2;

      UNTIL( ABS( temp - prev ) < prec );

      asin := temp;

    END;


{ ---------------------------------------------------------------------------------------------------------- acos ----- }
  FUNCTION acos( x : REAL ) : REAL;
    VAR
      temp, prev : REAL;
      num, den   : LONGINT;
      i, ni, di  : BYTE;

    BEGIN
      temp := PI / 2 - x;
      i := 3;
      
      num := 1; { Numerator }
      ni  := 1;
      den := 1; { Denominator }
      di  := 2;
      
      REPEAT
        prev := temp;
        num := num * ni;
        den := den * di;
        temp := temp - EXP( i * LN( x ) )* num / den / ( di + 1 );
        i := i + 2;
        ni := ni + 2;
        di := di + 2;

      UNTIL( ABS( temp - prev ) < prec );

      acos := temp;

    END;


{ ---------------------------------------------------------------------------------------------------------- atan ----- }
  FUNCTION atan( x : REAL ) : REAL;
    VAR
      temp, prev : REAL;
      sign : INTEGER;
      i : BYTE;

    BEGIN
      temp := 0;
      sign := 1;
      i := 1;

      REPEAT
        prev := temp;
        temp := temp + sign * EXP( i * LN( x ) ) / i;
        sign := -sign;
        i := i + 2;

      UNTIL( ABS( temp - prev ) < prec );

      atan := temp;

    END;


{ ---------------------------------------------------------------------------------------------------------- acot ----- }
  FUNCTION acot( x : REAL ) : REAL;
    VAR
      temp, prev : REAL;
      sign : INTEGER;
      i : BYTE;

    BEGIN
      temp := PI / 2;
      sign := -1;
      i := 1;

      REPEAT
        prev := temp;
        temp := temp + sign * EXP( i * LN( x ) ) / i;
        sign := -sign;
        i := i + 2;

      UNTIL( ABS( temp - prev ) < prec );

      acot := temp;

    END;


{ ----------------------------------------------------------------------------------------------------------- log ----- }
  FUNCTION log( x, a : REAL ) : REAL;
    BEGIN
      log := LN( x ) / LN( a );

    END;


{ --------------------------------------------------------------------------------------------------------- log10 ----- }
  FUNCTION log10( x : REAL ) : REAL;
    BEGIN
      log10 := LN( x )* 0.434294481;

    END;


{ ----------------------------------------------------------------------------------------------------------- pow ----- }
  FUNCTION pow( a, x : REAL ) : REAL;
    BEGIN
      pow := EXP( x * LN( a ) );

    END;


{ ----------------------------------------------------------------------------------------------------------- fac ----- }
  FUNCTION fac( x : REAL ) : REAL;
    BEGIN
      IF( x = 0 )THEN
        fac := 0
      ELSE { 0 < x < 13 }
        fac := SQRT( 2 * PI * x )* EXP( x * LN( x ) ) * EXP( -x + 1 / 12 / x );

    END;


{ ------------------------------------------------------------------------------------------------------- sigmoid ----- }
  FUNCTION sigmoid( x, a : REAL ) : REAL;
    BEGIN
      sigmoid := 1 / ( 1 + EXP( -a * x ) );

    END;


{ ---------------------------------------------------------------------------------------------------------- sinh ----- }
  FUNCTION sinh( x : REAL ) : REAL;
    BEGIN
      sinh :=( EXP( x )- EXP( -x ) ) / 2;

    END;


{ ---------------------------------------------------------------------------------------------------------- cosh ----- }
  FUNCTION cosh( x : REAL ) : REAL;
    BEGIN
      cosh :=( EXP( x )+ EXP( -x ) ) / 2;

    END;


{ ---------------------------------------------------------------------------------------------------------- tanh ----- }
  FUNCTION tanh( x : REAL ) : REAL;
    VAR
      p, m : REAL;

    BEGIN
      p := EXP(  x );
      m := EXP( -x );
      tanh :=( p - m ) / ( p + m );

    END;


{ ---------------------------------------------------------------------------------------------------------- coth ----- }
  FUNCTION coth( x : REAL ) : REAL;
    VAR
      p, m : REAL;

    BEGIN
      p := EXP(  x );
      m := EXP( -x );
      coth :=( p + m ) / ( p - m );

    END;


{ ---------------------------------------------------------------------------------------------------------- INIT ----- }
BEGIN END .